-------------------------------------------------------------------------------
--
-- NIOS Wrapper for AES Decrypto Module
-- Written for 128-bit AES decryption project
-- Course: CSEE 4840 - Embedded System Design, Spring 2008
-- Authors: Shrivathsa Bhargav (sb2784)
--			Larry Chen (lc2454)
--			Abhinandan Majumdar (am2993)
--			Shiva Ramudit (syr9)
--
-- Last modified: 5-8-2008
-------------------------------------------------------------------------------

library IEEE;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;

--Wrapper Module for AES Decrypto Module with NIOS via Avalon Bus
entity aes128_nios is 
port (
clk 		: in std_logic; 
reset 	: in std_logic;
read		: in std_logic;
write		: in std_logic;
chipselect	: in std_logic;
address		: in std_logic_vector (2 downto 0); --Equivalent to enable lines. address(0) for demuxing Cipher-Text and Key-Data. address(1) for enabling Key Expansion. address(2) for enabling AES Decrypto
readdata	: out std_logic_vector (31 downto 0);
writedata   : in std_logic_vector (31 downto 0)	
);
end aes128_nios;

architecture aes128_nios_arch of aes128_nios is

component AES_decrypto is
port (
ci 					   : in std_logic_vector(127 downto 0);
clk			     		   : in std_logic;
EN					   : in std_logic_vector(2 downto 0);
pl 					   : out std_logic_vector(127 downto 0);
eoc					   : out std_logic
);
end component AES_decrypto;

signal ci : std_logic_vector(127 downto 0);
signal en : std_logic_vector(2 downto 0);
signal pl : std_logic_vector(127 downto 0);
signal eoc : std_logic;

begin


process (clk) 
begin
if rising_edge(clk) then
	if reset = '1' then
	   readdata <= (others => '0');
	   en <= "000";
	   ci <= (others => '0');
	else
		if chipselect = '1' then
			--Sends 128 bit data in 32 bit chunk.
			--After sending 4th 32 bit chunk, it starts AES Decrypton or Key Expansion based upon Address Line
			if write = '1' then
				if address(1 downto 0) = "00" then 
					ci(127 downto 96) <= writedata;
					if address(2) = '0' then
						en <= "000";
					elsif address(2) = '1' then
						en <= "100";
					end if;
				elsif address(1 downto 0) = "01" then
					ci(95 downto 64) <= writedata;
					if address(2) = '0' then
					en <= "000";
					elsif address(2) = '1' then
					en <= "100";
					end if;
				elsif address(1 downto 0) = "10" then
					ci(63 downto 32) <= writedata;
					if address(2) = '0' then
					en <= "000";
					elsif address(2) = '1' then
					en <= "100";
					end if;
				elsif address(1 downto 0) = "11" then
					ci(31 downto 0) <= writedata;
					if address(2) = '0' then
					en <= "001";
					elsif address(2) = '1' then
					en <= "110";
					end if;
				end if;
			elsif read = '1' then
				--When eoc = '0' sends the original cipher text/key data being sent.
				if eoc = '0' then
					if address(1 downto 0) = "00" then 
						readdata <= ci(127 downto 96);
					elsif address(1 downto 0) = "01" then
						readdata <= ci(95 downto 64);
					elsif address(1 downto 0) = "10" then
						readdata <= ci(63 downto 32);
					elsif address(1 downto 0) = "11" then
						readdata <= ci(31 downto 0);
					else
						readdata <= x"ABCD1234";
					end if;
				--When eoc = '1', sends the plain text (after decryption) in 32 bit chunks based upon the address line.
				elsif eoc = '1' then
					if address(1 downto 0) = "00" then 
						readdata <= pl(127 downto 96);
					elsif address(1 downto 0) = "01" then
						readdata <= pl(95 downto 64);
					elsif address(1 downto 0) = "10" then
						readdata <= pl(63 downto 32);
					elsif address(1 downto 0) = "11" then
						readdata <= pl(31 downto 0);
					end if;
				end if;
			end if;
		else
			readdata <= (others => '1');
		end if;
	end if;
end if;
end process;

--AES Decrypto Central Module
aes128: AES_decrypto port map(
ci => ci,
clk => clk, en => en,
pl => pl,
eoc => eoc
);



end aes128_nios_arch;
