-------------------------------------------------------------------------------
--
-- DE2 top-level module for AES decryption project
-- Written for 128-bit AES decryption project
-- Course: CSEE 4840 - Embedded System Design, Spring 2008
-- Authors: Shrivathsa Bhargav (sb2784)
--			Larry Chen (lc2454)
--			Abhinandan Majumdar (am2993)
--			Shiva Ramudit (syr9)
--
-- Last modified: 5-8-2008
-------------------------------------------------------------------------------
-------------------------------------------------------------------------------
--
-- DE2 top-level module for the Apple ][
--
-- Stephen A. Edwards, Columbia University, sedwards@cs.columbia.edu
--
-- From an original by Terasic Technology, Inc.
-- (DE2_TOP.v, part of the DE2 system board CD supplied by Altera)
--
-------------------------------------------------------------------------------


library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;

entity AES128_toplevel is

  port (
    -- Clocks
    
    CLOCK_27,                                      -- 27 MHz
    CLOCK_50,                                      -- 50 MHz
    EXT_CLOCK : in std_logic;                      -- External Clock

    -- Buttons and switches
    
    KEY : in std_logic_vector(3 downto 0);         -- Push buttons
    SW : in std_logic_vector(17 downto 0);         -- DPDT switches

    -- LED displays

    HEX0, HEX1, HEX2, HEX3, HEX4, HEX5, HEX6, HEX7 -- 7-segment displays
    : out unsigned(6 downto 0);
    signal LEDG : out std_logic_vector(8 downto 0);       -- Green LEDs
    LEDR : out std_logic_vector(17 downto 0);      -- Red LEDs

    -- RS-232 interface

    UART_TXD : out std_logic;                      -- UART transmitter   
    UART_RXD : in std_logic;                       -- UART receiver

    -- IRDA interface

    -- IRDA_TXD : out std_logic;                      -- IRDA Transmitter
    IRDA_RXD : in std_logic;                       -- IRDA Receiver

    -- SDRAM
    
    DRAM_DQ : inout std_logic_vector(15 downto 0); -- Data Bus
    DRAM_ADDR : out std_logic_vector(11 downto 0); -- Address Bus    
    DRAM_LDQM,                                     -- Low-byte Data Mask 
    DRAM_UDQM,                                     -- High-byte Data Mask
    DRAM_WE_N,                                     -- Write Enable
    DRAM_CAS_N,                                    -- Column Address Strobe
    DRAM_RAS_N,                                    -- Row Address Strobe
    DRAM_CS_N,                                     -- Chip Select
    DRAM_BA_0,                                     -- Bank Address 0
    DRAM_BA_1,                                     -- Bank Address 0
    DRAM_CLK,                                      -- Clock
    DRAM_CKE : out std_logic;                      -- Clock Enable

    -- FLASH
    
    FL_DQ : inout std_logic_vector(7 downto 0);      -- Data bus
    FL_ADDR : out std_logic_vector(21 downto 0);  -- Address bus
    FL_WE_N,                                         -- Write Enable
    FL_RST_N,                                        -- Reset
    FL_OE_N,                                         -- Output Enable
    FL_CE_N : out std_logic;                         -- Chip Enable

    -- SRAM
    
    SRAM_DQ : inout std_logic_vector(15 downto 0);         -- Data bus 16 Bits
    SRAM_ADDR : out std_logic_vector(17 downto 0);         -- Address bus 18 Bits
    SRAM_UB_N,                                     -- High-byte Data Mask 
    SRAM_LB_N,                                     -- Low-byte Data Mask 
    SRAM_WE_N,                                     -- Write Enable
    SRAM_CE_N,                                     -- Chip Enable
    SRAM_OE_N : out std_logic;                     -- Output Enable

    -- USB controller
    
    OTG_DATA : inout std_logic_vector(15 downto 0); -- Data bus
    OTG_ADDR : out std_logic_vector(1 downto 0);    -- Address
    OTG_CS_N,                                       -- Chip Select
    OTG_RD_N,                                       -- Write
    OTG_WR_N,                                       -- Read
    OTG_RST_N,                                      -- Reset
    OTG_FSPEED,                     -- USB Full Speed, 0 = Enable, Z = Disable
    OTG_LSPEED : out std_logic;     -- USB Low Speed, 0 = Enable, Z = Disable
    OTG_INT0,                                       -- Interrupt 0
    OTG_INT1,                                       -- Interrupt 1
    OTG_DREQ0,                                      -- DMA Request 0
    OTG_DREQ1 : in std_logic;                       -- DMA Request 1   
    OTG_DACK0_N,                                    -- DMA Acknowledge 0
    OTG_DACK1_N : out std_logic;                    -- DMA Acknowledge 1

    -- 16 X 2 LCD Module
    
    LCD_ON,                     -- Power ON/OFF
    LCD_BLON,                   -- Back Light ON/OFF
    LCD_RW,                     -- Read/Write Select, 0 = Write, 1 = Read
    LCD_EN,                     -- Enable
    LCD_RS : out std_logic;     -- Command/Data Select, 0 = Command, 1 = Data
    LCD_DATA : inout std_logic_vector(7 downto 0); -- Data bus 8 bits

    -- SD card interface
    
    SD_DAT : in std_logic;      -- SD Card Data      SD pin 7 "DAT 0/DataOut"
    SD_DAT3 : out std_logic;    -- SD Card Data 3    SD pin 1 "DAT 3/nCS"
    SD_CMD : out std_logic;     -- SD Card Command   SD pin 2 "CMD/DataIn"
    SD_CLK : out std_logic;     -- SD Card Clock     SD pin 5 "CLK"

    -- USB JTAG link
    
    TDI,                        -- CPLD -> FPGA (data in)
    TCK,                        -- CPLD -> FPGA (clk)
    TCS : in std_logic;         -- CPLD -> FPGA (CS)
    TDO : out std_logic;        -- FPGA -> CPLD (data out)

    -- I2C bus
    
    I2C_SDAT : inout std_logic; -- I2C Data
    I2C_SCLK : out std_logic;   -- I2C Clock

    -- PS/2 port

    PS2_DAT,                    -- Data
    PS2_CLK : in std_logic;     -- Clock

    -- VGA output
    
    VGA_CLK,                                            -- Clock
    VGA_HS,                                             -- H_SYNC
    VGA_VS,                                             -- V_SYNC
    VGA_BLANK,                                          -- BLANK
    VGA_SYNC : out std_logic;                           -- SYNC
    VGA_R,                                              -- Red[9:0]
    VGA_G,                                              -- Green[9:0]
    VGA_B : out std_logic_vector(9 downto 0);           -- Blue[9:0]

    --  Ethernet Interface
    
    ENET_DATA : inout std_logic_vector(15 downto 0);    -- DATA bus 16Bits
    ENET_CMD,           -- Command/Data Select, 0 = Command, 1 = Data
    ENET_CS_N,                                          -- Chip Select
    ENET_WR_N,                                          -- Write
    ENET_RD_N,                                          -- Read
    ENET_RST_N,                                         -- Reset
    ENET_CLK : out std_logic;                           -- Clock 25 MHz
    ENET_INT : in std_logic;                            -- Interrupt
    
    -- Audio CODEC
    
    AUD_ADCLRCK : inout std_logic;                      -- ADC LR Clock
    AUD_ADCDAT : in std_logic;                          -- ADC Data
    AUD_DACLRCK : inout std_logic;                      -- DAC LR Clock
    AUD_DACDAT : out std_logic;                         -- DAC Data
    AUD_BCLK : inout std_logic;                         -- Bit-Stream Clock
    AUD_XCK : out std_logic;                            -- Chip Clock
    
    -- Video Decoder
    
    TD_DATA : in std_logic_vector(7 downto 0);  -- Data bus 8 bits
    TD_HS,                                      -- H_SYNC
    TD_VS : in std_logic;                       -- V_SYNC
    TD_RESET : out std_logic;                   -- Reset
   
    -- General-purpose I/O
    
    GPIO_0,                                      -- GPIO Connection 0
    GPIO_1 : inout std_logic_vector(35 downto 0) -- GPIO Connection 1   
    );
  
end AES128_toplevel;

architecture AES128_toplevel_arch of AES128_toplevel is

  component CLK14MPLL is
    port (
      inclk0    : in std_logic;
      c0        : out std_logic);
  end component;

  component sdram_pll is
	port (
	  inclk0	: in std_logic;
	  c0		: out std_logic);
  end component;

  signal CLK_14M, CLK_2M, PRE_PHASE_ZERO : std_logic;
  signal IO_SELECT, DEVICE_SELECT : std_logic_vector(7 downto 0);
  signal ADDR : unsigned(15 downto 0);
  signal D, PD : unsigned(7 downto 0);

  signal ram_we : std_logic;
  signal VIDEO, HBL, VBL, LD194 : std_logic;
  signal COLOR_LINE : std_logic;
  signal COLOR_LINE_CONTROL : std_logic;
  signal GAMEPORT : std_logic_vector(7 downto 0);
  signal cpu_pc : unsigned(15 downto 0);

  signal K : unsigned(7 downto 0);
  signal read_key : std_logic;

  signal flash_clk : unsigned(22 downto 0);
  signal reset : std_logic;

  signal track : unsigned(5 downto 0);
  signal trackmsb : unsigned(3 downto 0);
  signal D1_ACTIVE, D2_ACTIVE : std_logic;
  signal track_addr : unsigned(13 downto 0);
  signal TRACK_RAM_ADDR : std_logic_vector(13 downto 0);
  signal tra : unsigned(15 downto 0);
  signal TRACK_RAM_DI : std_logic_vector(7 downto 0);
  signal TRACK_RAM_WE : std_logic;

  signal CS_N, MOSI, MISO, SCLK : std_logic;

  signal SLOW_CLK : std_logic;		-- Larry
  signal clk25 : std_logic := '0';	-- B

  signal temp : unsigned(3 downto 0) := "0000";
  signal temp2 : unsigned(3 downto 0) := "0000";
  signal temp3 : unsigned(3 downto 0) := "0000";
  signal debug : unsigned(7 downto 0) := "00000000";
  signal counter3 : unsigned (2 downto 0) := "000"; 
  signal counter2 : unsigned (2 downto 0) := "000"; 

  signal addr_input : std_logic_vector(31 downto 0);
  signal size_input : std_logic_vector(31 downto 0);
  signal start_input : std_logic := '0';

  type ram_type is array (7 downto 0) of unsigned(7 downto 0);
  signal RAM : ram_type := ( x"00", 
 x"00", 
 x"00", 
 x"00", 
 x"00", 
 x"00", 
 x"00", 
 x"00");

begin

  reset <= SW(0);

  -- Clock divider for VGA
  process (CLOCK_50)
  begin
    if rising_edge(CLOCK_50) then
      clk25 <= not clk25;
    end if;
  end process;


  COLOR_LINE_CONTROL <= COLOR_LINE and SW(17);  -- Color or B&W mode

--Instantiate the sdram pll entity
neg_3ns : sdram_pll port map (CLOCK_50, DRAM_CLK);

nios : entity work.nios_system port map (
	  
      -- the_sdram
    zs_addr_from_the_sdram => DRAM_ADDR, 
    zs_ba_from_the_sdram(1) => DRAM_BA_1,
    zs_ba_from_the_sdram(0) => DRAM_BA_0,  
    zs_cas_n_from_the_sdram => DRAM_CAS_N,
    zs_cke_from_the_sdram => DRAM_CKE,
    zs_cs_n_from_the_sdram => DRAM_CS_N,
    zs_dq_to_and_from_the_sdram => DRAM_DQ,
    zs_dqm_from_the_sdram(1) => DRAM_UDQM,
    zs_dqm_from_the_sdram(0) => DRAM_LDQM,
    zs_ras_n_from_the_sdram => DRAM_RAS_N,
    zs_we_n_from_the_sdram => DRAM_WE_N,

      -- VGA signals
    VGA_BLANK_from_the_vgasram	=> VGA_BLANK,
    VGA_B_from_the_vgasram		=> VGA_B,
    VGA_CLK_from_the_vgasram	=> VGA_CLK,
    VGA_G_from_the_vgasram		=> VGA_G,
    VGA_HS_from_the_vgasram		=> VGA_HS,
    VGA_R_from_the_vgasram		=> VGA_R,
    VGA_SYNC_from_the_vgasram	=> VGA_SYNC,
    VGA_VS_from_the_vgasram		=> VGA_VS,  
    --go_to_the_vgasram => SW(0),

    -- the_lcd
     LCD_BLON_from_the_character_lcd => LCD_BLON,
     LCD_DATA_to_and_from_the_character_lcd => LCD_DATA,
     LCD_EN_from_the_character_lcd => LCD_EN,
     LCD_ON_from_the_character_lcd => LCD_ON,
     LCD_RS_from_the_character_lcd => LCD_RS,
     LCD_RW_from_the_character_lcd => LCD_RW,

    -- the_keyboard
    PS2_Clk_to_the_keyboard => PS2_CLK,
    PS2_Data_to_the_keyboard => PS2_DAT,

	
    SRAM_ADDR_from_the_vgasram => SRAM_ADDR,
    SRAM_CE_N_from_the_vgasram => SRAM_CE_N,
    SRAM_DQ_to_and_from_the_vgasram => SRAM_DQ,
    SRAM_LB_N_from_the_vgasram => SRAM_LB_N,
    SRAM_OE_N_from_the_vgasram => SRAM_OE_N,
    SRAM_UB_N_from_the_vgasram => SRAM_UB_N,
    SRAM_WE_N_from_the_vgasram => SRAM_WE_N,
    CS_N_from_the_spi => CS_N,
    MOSI_from_the_spi => MOSI,
    SCLK_from_the_spi => SCLK,
    MISO_to_the_spi => MISO,
    clk => CLOCK_50,
    reset_n => '1'
    );

  SD_DAT3 <= CS_N;
  SD_CMD  <= MOSI;
  MISO    <= SD_DAT;
  SD_CLK  <= SCLK;

  LEDG(0) <= SW(0);

end AES128_toplevel_arch;
